 %LoadFromSpec.m
 
function LoadFromSpec
global MSpScnList MSpf  MSpScn ImgFormatList;
global MotDetOff
persistent c0 cpd strChanFirst strChanLast strIncAng strLam
persistent strScanNum strFileName

if ~exist('MSpScnList')
   MSpScnList={};
end;
if isempty(MotDetOff),
	 MotDetOff={};
end;
if isempty(c0),
	c0=541;
end;
if isempty(cpd),
	cpd=268;
end;
if isempty(strChanFirst),
	strChanFirst='-';
end;
if isempty(strChanLast),
	strChanLast='-';
end;
if isempty(strIncAng),
	strIncAng='-';
end;
if isempty(strLam),
	strLam='1.55';
end;
if isempty(strScanNum),
	strScanNum='18';
end;
if isempty(strFileName),
	strFileName='MSpScanLiquidA.spec';
end;
 
f0=figure('Visible','off',...
	'MenuBar', 'none', ...
'Position',[400,400,500,400]);
set(f0,'tag','LoadFromSpec');
set(f0,'Name','MatSpec: Load from SPEC',...
	'CloseRequestFcn',{@closeIt});
set(f0,'Visible','on');
hndMainGUI=findobj('Tag','figGUIMatSpec');
set(hndMainGUI,'Visible','Off');


hLbSpecFileName = uicontrol('Style','text','String','SPEC file name:','Position',[25,350,100,25]);
hEdSpecFileName = uicontrol('Style','edit','String',strFileName,'Position',[125,350,250,25]);
hBtSpecFileName=uicontrol('Style','pushbutton','String','Browse','Position',...
	[375,350,100,25],'Callback',{@specFileName_Callback});


hLbScanNumber = uicontrol('Style','text','String','Scan Number:','Position',[25,320,100,25]);
hEdScanNumber = uicontrol('Style','edit','String',strScanNum,'Position',[125,320,150,25]);

hLbWaveLength= uicontrol('Style','text','String','WaveLength:','Position',[25,290,100,25]);
hEdWaveLength = uicontrol('Style','edit','String',strLam,'Position',[125,290,150,25]);

hPnDetector= uicontrol('Style','frame','Position',[20,240,460,45]);
%set(hPnDetector,'Enable','off');
hLDetector= uicontrol('Style','text','String','1D detector info:','Position',[30,250,100,25]);
hLChannel= uicontrol('Style','text','String','Central channel:','Position',[130,250,100,25]);
hEdChannel= uicontrol('Style','edit','String',num2str(c0,'%d'),'Position',[230,250,50,25]);
hLChannelDegrees= uicontrol('Style','text','String','Channels/degrees:','Position',[280,250,100,25]);
hEdChannelDegrees= uicontrol('Style','edit','String',num2str(cpd,'%.2f'),'Position',[380,250,50,25]);

hfrOptions= uicontrol('Style','frame','Position',[20,100,460,120]);
hLOptions= uicontrol('Style','text','String','----------------------OPTIONS------------------------','Position',[30,190,440,20]);
hLIncidenceangle= uicontrol('Style','text','String',['Incidence ',...
'angle (for a beam deflected by a X-Ray mirror):'],...
'Position',[30,160,300,25]);
hEdIncidenceangle= uicontrol('Style','edit','String',strIncAng,'Position',[380,160,50,25]);

hLDetectorPR= uicontrol('Style','text','String','1D detector info:','Position',[30,110,100,25]);
hLFirstPixel= uicontrol('Style','text','String','First Pixel:','Position',[130,110,100,25]);
hEdFirstPixel= uicontrol('Style','edit','String',strChanFirst,'Position',[230,110,50,25]);
hLLastPixel= uicontrol('Style','text','String','Last Pixel:','Position',[280,110,100,25]);
hEdLastPixel= uicontrol('Style','edit','String',strChanLast,'Position',[380,110,50,25]);

warning('off');
hLDataSetName= uicontrol('Style','text','String','Matlab data set name:','Position',[30,50,130,25]);
if ~isempty(MSpScnList),
	hPopDataSetName= uicontrol('Style','popupmenu','String',...
	MSpScnList, 'Position',[180,50,100,25],'Callback',...
	{@dataPopSetName_Callback});
else
	hPopDataSetName= uicontrol('Style','popupmenu','String',...
	{'-'}, 'Position',[180,50,100,25],'Callback',...
	{@dataPopSetName_Callback});
end;

hBtDataSetName=uicontrol('Style','pushbutton','String','New data set name',...
	'Position',[30,20,150,25],'Callback',{@btDataSetName_Callback});
hEdDataSetName= uicontrol('Style','edit','String','-','Position',[180,20,100,25]);

    
hBtSetMotorOffset=uicontrol('Style','pushbutton','String',...
	'Set Mot. offset','Position',[300,20,80,25],...
	'Callback',{@setMotorOffset_Callback});
hBtLoad=uicontrol('Style','pushbutton','String','Load',...
	'Position',[380,20,80,25],'Callback',{@load_Callback});
warning('on')

function specFileName_Callback(hObject,eventdata)
persistent LocalDir
if isempty(LocalDir),
 LocalDir='';
end;

if isempty(LocalDir),
  mypath='./*.*';
else
    mypath=[LocalDir,'*.*'];
end;
[file,mypath]=uigetfile(mypath);

%[file,mypath]=uigetfile('*.spec',mypath);

LocalDir=mypath;

set(hEdSpecFileName,'String',[mypath,file]);
end

function btDataSetName_Callback(source, eventdata)
	scn=get(hEdDataSetName,'String');
	if ~isvarname(scn),
		msgbox(sprintf('Warning: %s is not a valid dataset name',scn));
		return;
	end;
	if ~any(strcmp(MSpScnList,scn)),
		n=length(MSpScnList);
		MSpScnList{n+1}=scn;
	end;
	set(hPopDataSetName,'String',MSpScnList);
  ii=find(strcmp(scn,MSpScnList),1);
  set(hPopDataSetName,'Value',ii);
end

function dataPopSetName_Callback(source, eventdata)
    str=get(source,'String');
    val=get(source,'Value');
    scn=str{val};
    set(hEdDataSetName,'String',scn);
end

function setMotorOffset_Callback(hObject,eventdata)
    str=get(hPopDataSetName,'String');
    val=get(hPopDataSetName,'Value');
    MSpScn=str{val};
    SetMotorOffset;
%disp('setMotorOffset');
end

function load_Callback(hObject,eventdata)
% Variable declaration
nMSpScnList=length(MSpScnList);
cmd='';
PrintFormat='eps';
PrintExt='eps';
cscn=0;


flgUnspec=1; % Extract data from a SPEC file to Matlab
flgNormalize=0; % Normalize intensities for monitor and absorbers 
flgAng2Q=0; % Transform from the angular space to the reciprocal space
flgPlotPSD=0; % Plot rough PSD spectra
flgAutoGridEst=0; % Should I perform automatic estimation of the density
									%	and size of the grid?
flgGridQ=0;	% Perform gridding (mapping of data in Q space from unevenly
						% distributed measurement points to a regular grid)
flgPlotGrid=0; % Plot data in Q space (gridded data) 
flgPrintPlot=0; % Print plot currently plotted to a pdf file
flgSaveMat=0; % Save the scan data to a mat file (matlab native format)
flgLoadMat=0; % Load the scan data from a mat file previously saved
							% (speeds up the process of plotting, whenever extracting
							% and gridding of data was already done in a previous
							% session)
flgSaveAscii=0; % Export data into ASCII files 

% The below are retrived from GUI
SpecName=get(hEdSpecFileName,'String');
strFileName=SpecName;
strScanNum=get(hEdScanNumber,'String');
scNum=str2num(strScanNum);   

strChanFirst=get(hEdFirstPixel,'String');
firstPixel=str2num(strChanFirst);
strChanLast=get(hEdLastPixel,'String');
lastPixel=str2num(strChanLast);
if (isempty(firstPixel) | isempty(lastPixel)),
	ROI=[];
else,
	ROI=[firstPixel,lastPixel];
end;
strIncAng=get(hEdIncidenceangle,'String');
alfi=str2num(strIncAng);

str=get(hPopDataSetName,'String');
val=get(hPopDataSetName,'Value');
scn=str{val};
if ~isvarname(scn),
	warndlg(sprintf('Warning: %s is not a valid dataset name',scn));
	return;
end;

lbd=get(hEdWaveLength,'String');
lambda=str2num(lbd);

c0=get(hEdChannel,'String');
c0=str2num(c0);
cpd=get(hEdChannelDegrees,'String');
cpd=str2num(cpd);
strLam=get(hEdWaveLength,'String');
lambda=str2num(strLam);

%%%
if ~any(strcmp(MSpScnList,scn)),
	n=length(MSpScnList);
	MSpScnList{n+1}=scn;
end;
cmd=['global ',scn];
eval(cmd);
MSpScn=scn;

MSpMain2
set(hndMainGUI,'Visible','On');
cscn=eval(scn);
if isfield(cscn,'PSD'),
	msgbox(['Linear position sensitive detector data were loaded ',...
		'as variable PSD in ',...
		'the dataset ', MSpScn, ' and can be ploted or normalized.']);
else
	warndlg('Can not load linear position sensitive detector data.');
end;
close(f0);
end

function closeIt(gcbf,ar2);
set(hndMainGUI,'Visible','On');
delete(f0);
end;

end
 
 
 

