function scn= MSpAng2Q2(sco, NameChi, NameDel, NameGam)
% function scnn= MSpAng2Q2v2(scno,, NameChi, NameDel, NameGam)
% Transform angles to Q-coordinates for scattering geometry with 
% sample on horizontal stage at ID10B.
%	Sample surface plane makes angle chi with the horizontal plane 
%	(plane of the beam), detector arm is rotated around the vertical axis
%	by an angle del and  around the horizontal axis by angle gam, PSD
%	detector is oriented along the vertical axis for del=gam=0.
% Motor gam sits on the motor del.
%	For ID10B ESRF morotrs names: NameChi='chi', NameDel='delta',
%	NameGam='gamma', 
% (c) 2007, Jiri Novak, ESRF Grenoble, France
 
%%%%
% This program comes with ABSOLUTELY NO WARRANTY.  This is free
% software distributed under GNU licence, see
% http://www.gnu.org/licenses/ or the attached file GPLicense.txt



d2r=pi/180;
chi=sco.(NameChi)*d2r;
gam0=sco.(NameGam)*d2r;
del=sco.(NameDel)*d2r;
nChennels=size(sco.PSD,2);
cpd=sco.cpd;
c0=sco.c0;
channels=sco.channs;
nchi=length(chi);
ndel=length(del);
ngam0=length(gam0);
nmax=max([nchi,ndel,ngam0]);
if (nchi<nmax),
	if (nchi~=1) error('Wrong number of scan points for %s\n',NameChi);
	end;
	chi=repmat(chi,nmax,1);
end;
if (ndel<nmax),
	if (ndel~=1) error('Wrong number of scan points for %s\n',NameDel);
	end;
	del=repmat(del,nmax,1);
end;
if (ngam0<nmax),
	if (ngam0~=1) error('Wrong number of scan points for %s\n',NameGam);
	end;
	gam0=repmat(gam0,nmax,1);
end;
gamd=atan((channels-c0)*d2r/cpd);
csgam0=cos(gam0); sngam0=sin(gam0);
csgamd=cos(gamd); sngamd=sin(gamd);
csgam=csgam0*csgamd-sngam0*sngamd;
sngam=csgam0*sngamd+sngam0*csgamd;
csdel=cos(del);
sndel=sin(del);

kfx=zeros(size(csgam));
kfy=kfx; kfz=kfx;
for ii=1:nmax,
	kfx(ii,:)=csdel(ii)*csgam(ii,:);
	kfy(ii,:)=sndel(ii)*csgam(ii,:);
end;
kfz=sngam;
		
cschi=cos(chi); snchi=sin(chi);
if size(cschi,2)==1,
	cschi=repmat(cschi,1,nChennels);
	snchi=repmat(snchi,1,nChennels);
end;
Qx=kfx.*cschi+kfz.*snchi-cschi;
Qy=kfy;
Qz=kfz.*cschi-snchi.*kfx+snchi;
scn=sco;
K0=2*pi/sco.lam;
scn.Qx=Qx*K0;
scn.Qy=Qy*K0;
scn.Qz=Qz*K0;

