function s3=MSpConcatScans2(s1,s2,varargin);
% function s3=MSpConcatScans2(s1,s2,varargin);
% Concat structures corresponding to 2 scans to a 3rd structure.
% For numerical fields of the scan structures concat the motor positions
% or intensities in corresponding fields of s1 and s2 to a field in s3
% with the same name.
% If fields are structures or cell arrays returns a
% value for s1.
% Further pairs of arguments are accepted as next parameters
% 	'ConcatDir', n ... concat along dimesnion n, if not given,
% 		an automatical choice is done 
% Usage: s3=MSpConcatScans2(s1,s2,varargin);
% (c) 2007, Jiri Novak, ESRF Grenoble, France
 
%%%%
% This program comes with ABSOLUTELY NO WARRANTY.  This is free
% software distributed under GNU licence, see
% http://www.gnu.org/licenses/ or the attached file GPLicense.txt


narg=nargin;
ii=1;
inarg.fake=0;
while ii<narg-2,
	if ~isstr(varargin{ii}),
		error(['Usage: SpReadScan(fileStruc,scan_number,vargin1_string,',...
			'vargin1_value)']);
	end;
	inarg.(varargin{ii})=varargin{ii+1};
	ii=ii+2;
end;
if isfield(inarg,'ConcatDir'),
	ConcatDir=inarg.ConcatDir;
else
	ConcatDir=0;
end;
fnames=fieldnames(s1);
nf=length(fnames);
for ii=1:nf,
	cfn=fnames{ii};
	cfield=s1.(cfn);
	if isnumeric(cfield),
		if (ConcatDir==1 | (ConcatDir==0 & ...
			size(s1.(cfn),2)==size(s2.(cfn),2))),
			cfield=[s1.(cfn);s2.(cfn)];
		elseif (ConcatDir==2 | (ConcatDir==0 & ...
			size(s1.(cfn),1)==size(s2.(cfn),1))),
			cfield=[s1.(cfn),s2.(cfn)];
		else
			error('Can not concat scans\n');
		end;
	elseif isstr(cfield),
		cfield=[s1.(cfn),char(10),s2.(cfn)];
	end;
	s3.(cfn)=cfield;
end;
	
