% this is a version for GUI
% This script is a workhorse of the MatSpec2 package
% (c) 2007, Jiri Novak, ESRF Grenoble, France 
%%%%
% This program comes with ABSOLUTELY NO WARRANTY.  This is free
% software distributed under GNU licence, see
% http://www.gnu.org/licenses/ or the attached file GPLicense.txt


%!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!%
% Processing part of MatSpec2

% Make some variables global
% MSpf ... cell array of structures with SPEC-file structure information
% 	(sees MSpUnspec2) global - refer to them as MSpf{1}, Mspf{2} ...
% MSpScnList ... structure with names of structures with data on scans
% MSpScn ... current scan on which user is working on
if ~exist('MSpf','var'),
	eval(['global ','MSpf']);
end;
if ~exist('MSpScnList','var'),
	eval(['global ','MSpScnList']);
end;
if ~exist('MSpScn','var'),
	eval(['global ','MSpScn']);
end;

% global MSpf MSpScnList MSpScn

if (~iscell(MSpScnList)),
	MSpScnList={};
end;

if flgUnspec,
	nMSpScnList=length(MSpScnList);
	% Add name of the variable with scan data to the scan list MSpScnList,
	% only if it is not already there
	if ~any(strcmp(MSpScnList,MSpScn)),
		MSpScnList{nMSpScnList+1}=MSpScn;
	end;
	if ~isempty(ROI),
		cmd=[MSpScn,'=MSpUnspec2(SpecName,scNum,MotDetOff,''ROI'',ROI);'];
	else
		cmd=[MSpScn,'=MSpUnspec2(SpecName,scNum,MotDetOff);'];
	end;
	eval(cmd);
	cmd=[MSpScn,'=setfield(',MSpScn,',''lam'',lambda);'];
	eval(cmd);
	if exist('alfi'),
		cmd=[MSpScn,'.alfi=alfi;'];
		eval(cmd);
	end;
	if exist('c0'),
		if ~isempty(c0),
			cmd=[MSpScn,'.c0=c0;'];
			eval(cmd);
		end;
	end;
	if exist('cpd'),
		if ~isempty(cpd),
			cmd=[MSpScn,'.cpd=cpd;'];
			eval(cmd);
		end;
	end;
end;

if flgLoadMat,
	if ~strcmp('mat',FName(end-3:end)),
		FName=[FName,'.mat'];
	end;
	if ~exist(FName,'file'),
		error(['Can not open ',FName]);
	else
		load(FName);
		varlst=whos('-file',FName);
		MSpScn=varlst.name;
		nMSpScnList=length(MSpScnList);
		% Add name of the variable with scan data to the scan list MSpScnList,
		% only if it is not already there
		if ~any(strcmp(MSpScnList,MSpScn)),
			MSpScnList{nMSpScnList+1}=MSpScn;
		end;
	end;
	clear nMSpScnList varlst
end;

if flgNormalize,
	cmd=[MSpScn,'=','MSpNormalize2(',MSpScn,',AttFac,NameAtt,NameDet,',...
		'NameMon,MonNorm);'];
	eval(cmd);
end;

if flgPrintPlot,
	if ~isempty(findstr(PrintFormat,'eps')),
		PrintExt='eps';
	elseif ~isempty(findstr(PrintFormat,'ps')),
		PrintExt='ps';
	elseif ~isempty(findstr(PrintFormat,'jpeg')),
		PrintExt='jpg';
	elseif ~isempty(findstr(PrintFormat,'png')),
		PrintExt='png';
	elseif ~isempty(findstr(PrintFormat,'pdf')),
		PrintExt='pdf';
	end;
end;


if flgPlotPSD,
	hfig=findobj('tag','PSDImage');
	if isempty(hfig),
		hfig=figure;
		set(hfig,'tag','PSDImage');
	else
		figure(hfig);
	end;
	cmd=sprintf(['MSpPlotPSD2(%s, PSDPlotXField, PSDPlotPSD,'...
		'flgIntLog, ''truncLogTop'', %g, ''truncLogBot'', %g);'], ...
		MSpScn,truncLogTop,truncLogBot);
	eval(cmd);
	if flgPrintPlot,
		print(hfig,['-d',PrintFormat],[FName,'.',PrintExt]);
	end;
end;

if flgAng2Q,
	cmd=[MSpScn,'=',Ang2Q{1},'(',MSpScn,',Ang2Q{2:end});'];
	eval(cmd);
end;

if flgAutoGridEst,
	cmd=['[Qxg,delQx,Qyg,delQy,Qzg,delQz]=MSpEstimQGrid2(',...
		MSpScn,',xGridField,yGridField);'];
	eval(cmd);
end;
	
if flgGridQ,
	cmd=[MSpScn,'=MSpQGrid2(',MSpScn,',Qxg,delQx,Qyg,delQy,Qzg,delQz);'];
	eval(cmd);
end;


if flgPlotGrid,
	hfig=findobj('tag','QImage');
	if isempty(hfig),
		hfig=figure;
		set(hfig,'tag','QImage');
	else
		figure(hfig);
	end;
	eval(['sch=',MSpScn,';']);
	MSpPlotGrid2(sch,xGridField,yGridField,colorGridField,...
		flgIntLog,AxFontSize,...
		LabelFontSize,truncLogTop,truncLogBot);
	clear sch;
	if flgPrintPlot,
		print(['-d',PrintFormat],[FName,'Q.',PrintExt]);
	end;
end;

if flgSaveMat,
	save([FName,'.mat'],MSpScn);
end;

if flgSaveAscii,
	nit=length(SaveASCII_Vars);
	for ii=1:nit,
		cvar=SaveASCII_Vars{ii};
		cmd=['out=',MSpScn,'.(cvar);'];
		eval(cmd);
		if isnumeric(out),
			out=double(out);
		end;
		save([FName,'_',cvar,'.dat'],'out','-ascii','-tabs');
	end;	
	clear out ii nit cvar cmd
end

