%% MSpNormalize2.m
%% Normalizes intensity for absorbers and monitor (monitor is supposed
%% to be in front of the absorbers, othervise put AttFac=1)
%% 
%% function so=MSpNormalize2(si,AttFac,FieldNamFilterNr,...
%%		FieldNamDetectors,FieldNamMonitor,MonNorm);
%% si ... structure with a scan
%% AttFac ... mutiplicative factor: RealIntensity=AttFac^FilterNr*
%%		MeasuredIntensity if only scalar is given or 
%%		RealIntensity=MeasuredIntensity*AttFac(FilterNr) if AttFac is
%%    a vector with
%%		attenuation factors for filters 1:N
%% FieldNamFilterNr ... field with attenutaion factor for measurement
%%		points
%% FieldNamDetectors ... cell array of names of fields containing the
%%		detector's data.
%% FieldNamMonitor ... name of the motor with monitor
%% MonNorm ... factor by which should be all values normalized
%% so ... output structure with original measured data + 
%% 		fields with normalized data, normalized data are returned within
%% 		fields of the name {name of the original detector}"Norm"
%%
%% example:
%%		s54=MSpNormalize2(s54,3.1,'filtnr',{'pdetect','PSD'});
% (c) 2007, Jiri Novak, ESRF Grenoble, France 
%%%%
% This program comes with ABSOLUTELY NO WARRANTY.  This is free
% software distributed under GNU licence, see
% http://www.gnu.org/licenses/ or the attached file GPLicense.txt



function so=MSpNormaliz(si,AttFac,FieldNamFilterNr,...
	FieldNamDetectors,FieldNamMonitor,MonNorm);

so=si;
FNum=getfield(si,FieldNamFilterNr);
% Extend the attenuators for length of the scan if the attenuator did not
% 	change during the scan and thus it was reduced to a scalar by 
%		MSpUnspec
nAtt=length(FNum);
if iscellstr(FieldNamDetectors),
    CurName=FieldNamDetectors{1};
else
    CurName=FieldNamDetectors;
end;
x=getfield(si,CurName);
[nn,mm]=size(x);
if (nAtt==1 & nn>1)
	FNum=FNum*ones(nn,1);
end;
% Actual attenuation factors
if (length(AttFac)==1)
	ActMultFac=AttFac.^FNum;
else
	ActMultFac=ones(nn,1);
	ind=find(FNum>0);
	ActMultFac(ind)=AttFac(FNum(ind));
end;
% Correction for monitors
ActMultFac=ActMultFac./getfield(si,FieldNamMonitor)*MonNorm;
% Normalize all the detectors
for ii=1:length(FieldNamDetectors),
	CurName=FieldNamDetectors{ii};
	x=getfield(si,CurName);
	[nn,mm]=size(x);
	if (mm>1),
		ActMultFac1=repmat(ActMultFac,1,mm);
		xnorm=x.*ActMultFac1;
	else
		xnorm=x.*ActMultFac;
	end;
	so=setfield(so,[CurName,'Norm'],xnorm);
end;	
