%% MSpUnspec2.m
%%  script to import data from a spec file
%%  MCA spectra in separate mcafiles or in specfile
%%
%% Usage: [s, ROIout] = MSpUnspec2( SpecFile, ScanNbs, MotDetOff,varargin);
%% Input: SpecFile ... file name with root
%%		ScanNbs ... row vector of scan numbers of the scans, which 
%%			should be extracted
%%		 MotDetOff ... Cell array  conatining pairs of motor/detector
%%         names (string) and values to be substracted from
%%         motor/detector value given in the spec file
%%		 varargin ... further pairs of the input parrameters in the form 
%%         vargin1_1 'string', vargin1_2 value
%%         supported: 
%%                    'ROI', [100 300] for a PSD read channels 100-300
%%                           only
%%  Output:	
%%		s ... a cell array containing the motor positions, 
%%			intensities, PSD spectra 
%%    ROIout=[start_channel, end_channel] ... real range of channels
%%      read from the PSD
%%  Example usage:
%%        s15 = MSpUnspec('m63317.spec',[11 12],{});
%%        s15 = MSpUnspec('m63317.spec',[11 12],{'delta',3});
%%        [s15,rROI] = MSpUnspec('m63317.spec',[11 12],{'delta',3}, ...
%%              [100 300]);
%% .... by J. Novak (novak@esrf.fr), 08.11.2007;  
%%%%
% This program comes with ABSOLUTELY NO WARRANTY.  This is free
% software distributed under GNU licence, see
% http://www.gnu.org/licenses/ or the attached file GPLicense.txt


%%      (uses SpOpen2.m, SpReadScan.m)
% (c) 2007, Jiri Novak, ESRF Grenoble, France 
%%%%
% This program comes with ABSOLUTELY NO WARRANTY.  This is free
% software distributed under GNU licence, see
% http://www.gnu.org/licenses/ or the attached file GPLicense.txt


function [s, ROIout] = MSpUnspec2( SpecFile, ScanNbs, MotDetOff,varargin);

% Unpack varargin
ii=1;
flgROIDef=0;
while ii<nargin-3,
	invar.(varargin{ii})=varargin{ii+1};
	if strcmp(varargin{ii},'ROI'),
		flgROIDef=1;
	end;
	ii=ii+2;
end;

global MSpf  % List of already opened spec files 
if isempty(MSpf),
	MSpf={};
	nMSpf=0;
else
	nMSpf=length(MSpf);
end;

% Open the Spec file
% Is file already opened?
flgFOpened=0;
for ii=1:nMSpf,
	% Was the file SpecFile alread opened and readed?
	if strcmp(MSpf{ii}.fname,SpecFile),
		fstr=MSpf{ii};
		% Are we going to read last scan or a new scan about which 
		% there is no information in the fstr structure? yes=>rearead
		if (max(ScanNbs)>max(fstr.scnum)),
			fclose(fstr.fid);
			fstr=SpOpen2(SpecFile);
			MSpf{ii}=fstr;
		end;
		% Is file already opened?
		if ~isempty(fopen(fstr.fid)),
			flgFOpened=1;
			break;
		else
			fstr=SpOpen2(SpecFile);
			MSpf{ii}=fstr;
			flgFOpened=1;
		end;
	end;
end;
if ~flgFOpened,
	fstr=SpOpen2(SpecFile);
	MSpf{nMSpf+1}=fstr;
end;

ii=1;
for cSc=ScanNbs,
	if flgROIDef,
		stmp=SpReadScan2(fstr, cSc, 'ROI', invar.ROI,...
			'NoMotReduction',[],'NoPSDRotation',[]);
	else
		stmp=SpReadScan2(fstr, cSc,'NoMotReduction',[],'NoPSDRotation',[]); 
	end;
	if ii==1,
		s=stmp;
	else
		s=MSpConcatScans2(s,stmp,'ConcatDir',1);
	end;
	if isfield(stmp,'channs'),
		s.channs=stmp.channs;
	end;
	ii=ii+1;
end;	
s=SpReduceScan2(s);

if nargin>2,
	nMotDet=length(MotDetOff);
	ii=1;
	while ii<nMotDet,
		motdet=MotDetOff{ii};
		if ~isfield(s,motdet),
			error('Motor/detector %s not found',motdet);
		end;
		s.(motdet)=s.(motdet)-MotDetOff{ii+1};
		ii=ii+2;
	end;
end;

