function sc=SpReadScan2(fstr,scn,varargin);
% function sc=SpReadScan2(fstr,scn,varargin);
% Reads scan number scn from SPECfile, which was open by SpOpen and the
% returned structure is given in fstr. 
% Further pairs of arguments are accepted as next parameters
% 	'ROI', [pixmin pixmax]  range of pixels which should be extracted
% 	from MCA
%		'NoMotReduction' [] if defined, reduction of the postions of the not
%			moving motor to one number in field is not performed.
%		'NoPSDRotation' [] if defined, do not rotate PSD matrix so that
%			pixels are along columns
% Usage: sc=SpReadScan2(fileStruc,scan_number,vargin1_string, vargin1_value)
% Example: sc=SpReadScan2('nano.spec,15,'ROI', [50 100])
% (c) 2007, Jiri Novak, ESRF Grenoble, France
 
%%%%
% This program comes with ABSOLUTELY NO WARRANTY.  This is free
% software distributed under GNU licence, see
% http://www.gnu.org/licenses/ or the attached file GPLicense.txt


narg=nargin;
ii=1;
inarg.fake=0;
while ii<narg-2,
	if ~isstr(varargin{ii}),
		error(['Usage: SpReadScan2(fileStruc,scan_number,vargin1_string,',...
			'vargin1_value)']);
	end;
	inarg.(varargin{ii})=varargin{ii+1};
	ii=ii+2;
end;
nscn=length(fstr.scnpos);
ind=find(fstr.scnum==scn);
if isempty(ind),
	error('Cannot find scan %d in the file %s.\n', scn, fstr.fname);
end;
% Read head
fseek(fstr.fid,fstr.scnpos(ind),-1);
head=fread(fstr.fid,fstr.scnposEnd(ind)-fstr.scnpos(ind)+2,...
	'uchar=>uchar');
head=char(head');
sc.head=head;
% Read data
fseek(fstr.fid,fstr.scnposEnd(ind)+2,-1);
if (ind<nscn),
	nchar=fstr.scnpos(ind+1)-fstr.scnposEnd(ind)-1;
else
	nchar=Inf;
end;
datas=fread(fstr.fid,nchar, 'uchar=>uchar');
datas=char(datas');
% Extract motor positions at the beginning
MotPos=SpPurifyHeadChunk2(head,'#P\d+','\n');
MotPos=SpStr2cell(MotPos);
nMot=length(fstr.MotNames);
for ii=1:nMot,
	sc.(fstr.MotNames{ii})=MotPos{ii};
end;
% Extract motor and detector names at L header line if exist
if ~isempty(findstr(head,'#L')),
	MotLNames=SpPurifyHeadChunk2(head,'#L','\n');
	MotLNames=SpStr2cell(MotLNames);
else
	MotLNames={};
end;
% Contains MCA data?
flgMCA=findstr(head,'@MCA');
flgMCA=~isempty(flgMCA);
% Transfere data to matrices
[schunk,fchunk]=regexp(datas,'[^\\]\n');
nch=length(fchunk);
iif=0; ipsd=1; imot=1;
for ii=1:nch,
	iis=iif+1;
	iif=fchunk(ii);
	chunk=datas(iis:iif);
	chunk=regexprep(chunk,'[\s\\]+',' ');
	chunk=deblank(chunk);
	if (~isempty(findstr(chunk,'@A'))),
		datl=sscanf(chunk(3:end),'%g')';
		if (exist('PSD','var')),
			if (size(PSD,2)==length(datl)),
				PSD(ipsd,:)=datl;
			end;
		else
			PSD(ipsd,:)=datl;
		end;
		ipsd=ipsd+1;
	else
		datl=sscanf(chunk,'%g')';
		if (imot>1),
			if (size(motdet,2)==length(datl)),
				motdet(imot,:)=datl;
			end;
		else
			motdet(imot,:)=datl;
		end;
		imot=imot+1;
	end;
end;
imot=imot-1;
% Can we use single precission for data storage?
vers=version;
Mvers=sscanf(vers,'%d');
flgSinglePrec=0;
if  Mvers>6,
	flgSinglePrec=1;
	fprintf(1,['Matlab version > 7, MSpUnspec uses ',...
		'single precission for data storage.\n']);
end;

if (exist('PSD','var')),
	% single precision?
	ipsd=ipsd-1;
	if flgSinglePrec,
		PSD=single(PSD);
	end;
	% extract written channels-struncture: #@CHANN
	% number_of_channels_written 1st_channel_wr last_channel_wr
	% number_of_chann_binned_into_one
	[s,f]=regexp(head,'#@CHANN[^\n]*');
	chann=sscanf(head(s+7:f),'%d');
	% Number of channels binned into one "pixel" written to spec files
	% When 1 pixel correponds to 1 written pixel, sometimes 0 is returned
	% in #@CHANN 
	if chann(4)==0,
		chann(4)=1;
	end;
	channs=chann(2):chann(4):chann(3);
	if isfield(inarg,'ROI');
		ind=(channs>=inarg.ROI(1) & channs<=inarg.ROI(2));
		sc.channs=channs(ind);
		if ~isfield(inarg,'NoPSDRotation'),
			sc.PSD=PSD(:,ind)';
		else
			sc.PSD=PSD(:,ind);
		end;
	else
		sc.channs=channs;
		if ~isfield(inarg,'NoPSDRotation'),
			sc.PSD=PSD';
		else
			sc.PSD=PSD;
		end;
	end;
end;
% Attribute positions and intensities to motors and
% detectors, respectively
% single precision?
if flgSinglePrec & imot>0,
	motdet=single(motdet);
end;
nMot=length(MotLNames);
for ii=1:nMot,
	cmotdet=motdet(:,ii);
	if ~isfield(inarg,'NoMotReduction'),
		ma=max(cmotdet);
		mi=min(cmotdet);
		if (mi==ma),
			cmotdet=mi;
		end;
	end;
	sc.(MotLNames{ii})=cmotdet;
end;
if isfield(inarg,'NoMotReduction') & imot>0,
	fnames=fieldnames(sc);
	nf=length(fnames);
	for ii=1:nf,
		cfn=fnames{ii};
		if (size(sc.(cfn),1)==1 & isnumeric(sc.(cfn))),
			sc.(cfn)=repmat(sc.(cfn),imot,1);
		end;
	end;
end;
