/* gridq1_01.cc 
 * Grids data in Qx, Qy, Qz
 * Usage: gridq1_01 parameter_file ... grid data, use informations in
 * 	the parameter file
 * Usage: gridq1_01 -h ... write help
 *
 *
 * Follows an example input file ex.inp:
#input for gridq1_01
# Gridding in 3d by using averiging in bins or nearest neighbour method,
# Input data should be in binaries files saved as array of real*4 (float
# 32bit) numbers
# Gridded intensity is ordered so that qx changes most quickly, qy
# slowlier and qz slowlist so that grid grid_intensity (k*nx*ny+j*nx+i)
#  correspondes to the grid point [qxg(i) qyg(j) qzg(k)]
$17889 #number of datapoints
$real*4 qxtmp.b 0.000000 0.202020 100 0.200000  # file_with_qx qx_start delta-qx qx_bin_width
$real*4 qytmp.b 3.000000 0.105263 20 0.100000  # file_with_qy qy_start delta-qy qy_bin_width
$real*4 qztmp.b 10.100000 1.000000 1 0.100000  # file_with_qz qz_start delta-qz qz_bin_width
$real*4 ointtmp.b #input intensity
$real*4 ginttmp.b #gridded intensity, output
$method nearest #method of gridding: mean/nearest

(c) 2007, Jiri Novak, ESRF Grenoble, France
*/
#define dbg 0

#include<stdio.h>
#include<math.h>
#include<string.h>

inline void getind(float xmsdel,float hwdel,int n, int &imin,
    int &imax);
int nentry(FILE *pfile);

int main(int argc, char *argv[])
{
  char fname[50];
  char datatype[20];
  int nx,ny,nz,ximin,ximax,yimin,yimax,zimin,zimax;
  int ndat,i,j,k,l,ind,fmethod;
  int *ninten;
  float xmsdel,xhwdel,yhwdel,zhwdel,xdel,ydel,zdel,xs,ys,zs; 
  float xhw,yhw,zhw,fout,dx,dy,dz,dqq,qxg,qyg,qzg;
  float *inten,*qx,*qy,*qz,*distNearest;
  double *ginten;
  FILE *afile, *bfile;

                        
  if (argc!=2)
  {
    fprintf(stderr,"Error\nUsage: gridq1_01 input_file_name\n\
or gridq1_01 -h for help\n");
    return 1;
  }
        if (strcmp(argv[1],"-h")==0)
        {
					printf("\n****\ngridq1_01 ... Program for gridding data in 3D space\n***\n");
					printf("(c) 2007, Jiri Novak, ESRF Grenoble, France\n");
					printf("This program comes with ABSOLUTELY NO WARRANTY.  This is free\n\
software distributed under GNU licence, see\n\
http://www.gnu.org/licenses/ or the attached file GPLicense.txt\n\n");
                printf("Usage: gridq1_01 parameter_file\n\n\
Follows an example input file ex.inp:\n\
#input for gridq1_01\n\
# Gridding in 3d by using averiging in bins or nearest neighbour method,\n\
# Input data should be in binaries files saved as array of real*4 (float\n\
# 32bit) numbers\n\
# Gridded intensity is ordered so that qx changes most quickly, qy\n\
# slowlier and qz slowlist so that grid grid_intensity (k*nx*ny+j*nx+i)\n\
#  correspondes to the grid point [qxg(i) qyg(j) qzg(k)]\n\
$17889 #number of datapoints\n\
$real*4 qxtmp.b 0.000000 0.202020 100 0.200000  # file_with_qx qx_start delta-qx qx_bin_width\n\
$real*4 qytmp.b 3.000000 0.105263 20 0.100000  # file_with_qy qy_start delta-qy qy_bin_width\n\
$real*4 qztmp.b 10.100000 1.000000 1 0.100000  # file_with_qz qz_start delta-qz qz_bin_width\n\
$real*4 ointtmp.b #input intensity\n\
$real*4 ginttmp.b #gridded intensity, output\n\
$method nearest #method of gridding: mean/nearest\n");
                return 0;
        }
#if dbg==1  
  printf("Processing input file %s\n",argv[1]);
#endif
  /*Initalization*/
  afile=fopen(argv[1],"rt");
  if (afile==NULL){
    fprintf(stderr,"Can not open input file %s\n", argv[1]);
    return 1;
  }
  /*qx*/
  nentry(afile);
  fscanf(afile,"%d",&ndat);
#if dbg==1  
  printf("Number of datapoints %d\n",ndat);
#endif
  inten= new float[ndat]; 
  qx= new float[ndat];
  qy= new float[ndat];
  qz= new float[ndat];
  nentry(afile);
  fscanf(afile,"%s %s %f %f %d %f",datatype,fname,&xs,&xdel,&nx,&xhw);  
  if (!strcmp("real*4",datatype))
  {
    bfile=fopen(fname,"rb");
    if (bfile==NULL) {
      fprintf(stderr,"Can not open input file %s\n", fname);
      return 1;
    }
    for (i=0;i<ndat;i++) fread(qx+i,sizeof(float),1,bfile);
    fclose(bfile);
  }
  else {
    fprintf(stderr,"Unknow datatype %s\n", datatype);
    return 1;
  }
  xhw=xhw/2.;
  xhwdel=xhw/xdel;
  /*qy*/
  nentry(afile);
  fscanf(afile,"%s %s %f %f %d %f",datatype,fname,&ys,&ydel,&ny,&yhw);  
  if (!strcmp("real*4",datatype))
  {
    bfile=fopen(fname,"rb");
    if (bfile==NULL) {
      fprintf(stderr,"Can not open input file %s\n", fname);
      return 1;
    }
    for (i=0;i<ndat;i++) fread(qy+i,sizeof(float),1,bfile);
    fclose(bfile); 
  }
  else {
    fprintf(stderr,"Unknow datatype %s\n", datatype);
    return 1;
  }
  yhw=yhw/2.;
  yhwdel=yhw/ydel;
  /*qz*/
  nentry(afile);
  fscanf(afile,"%s %s %f %f %d %f",datatype,fname,&zs,&zdel,&nz,&zhw);  
  if (!strcmp("real*4",datatype))
  {
    bfile=fopen(fname,"rb");
    if (bfile==NULL) {
      fprintf(stderr,"Can not open input file %s\n", fname);
      return 1;
    }
    for (i=0;i<ndat;i++) fread(qz+i,sizeof(float),1,bfile);
    fclose(bfile); 
  }
  else {
    fprintf(stderr,"Unknow datatype %s\n", datatype);
    return 1;
  }
  zhw=zhw/2.;
  zhwdel=zhw/zdel;
  /*intensity*/
  nentry(afile);
  fscanf(afile,"%s %s",datatype,fname);  
  if (!strcmp("real*4",datatype))
  {
    bfile=fopen(fname,"rb");
    if (bfile==NULL) {
      fprintf(stderr,"Can not open input file %s\n", fname);
      return 1;
    }
    for (i=0;i<ndat;i++) fread(inten+i,sizeof(float),1,bfile);
    fclose(bfile); 
  }
  else {
    fprintf(stderr,"Unknow datatype %s\n", datatype);
    return 1;
  }
  nentry(afile);
  fscanf(afile,"%s %s",datatype,fname);
  if (!strcmp("real*4",datatype))
  {
    bfile=fopen(fname,"wb");
    if (bfile==NULL) {
      fprintf(stderr,"Can not open input file %s\n", fname);
      return 1;
    }
  }
  else {
    fprintf(stderr,"Unknow datatype %s\n", datatype);
    return 1;
  }
  nentry(afile);
  fscanf(afile,"%s",datatype);  
  if (!strcmp("mean",datatype)){
                fmethod=1;
        }
        else if (!strcmp("nearest",datatype)){
                fmethod=2;
        }
        else {
    fprintf(stderr,"Unknow averiging method: %s\n", datatype);
    return 1;
        }
  fclose(afile);
        /* Initialize variables for gridding */
        ginten=new double[nx*ny*nz];
  for (i=0;i<nx*ny*nz;i++) ginten[i]=0;
        if (fmethod==1){
                ninten=new int[nx*ny*nz];
        for (i=0;i<nx*ny*nz;i++) ninten[i]=0;
        }
        else if (fmethod==2){
                distNearest=new float[nx*ny*nz];
                for (i=0;i<nx*ny*nz;i++){
                        distNearest[i]=1.0e38;
                }
        }
    /* Grid data*/
  for (i=0;i<ndat;i++){
    xmsdel=(qx[i]-xs)/xdel;
    getind(xmsdel,xhwdel,nx,ximin,ximax);
    xmsdel=(qy[i]-ys)/ydel;
    getind(xmsdel,yhwdel,ny,yimin,yimax);
    xmsdel=(qz[i]-zs)/zdel;
    getind(xmsdel,zhwdel,nz,zimin,zimax);
    for (j=zimin;j<=zimax;j++){
      for (k=yimin;k<=yimax;k++){
        for (l=ximin;l<=ximax;l++){
          ind=j*ny*nx+k*nx+l;
          if (fmethod==1){
                                                ninten[ind]+=1;
                ginten[ind]+=inten[i];
                                        }
                                        else if (fmethod==2) {
                                                qxg=xs+xdel*(float)l;
                                                dx=qx[i]-qxg;
                                                qyg=ys+ydel*(float)k;
                                                dy=qy[i]-qyg;
                                                qzg=zs+zdel*(float)j;
                                                dz=qz[i]-qzg;
                                          dqq=dx*dx+dy*dy+dz*dz;                
                                                if (dqq<distNearest[ind]) {
                                                        distNearest[ind]=dqq;
                                                        ginten[ind]=inten[i];
                                                }
                                        }
        }
      }
    }
  }
  delete [] qx;
  delete [] qy;
  delete [] qz;
  delete [] inten;
  if (fmethod==1) {
                for (i=0;i<nx*ny*nz;i++){
                        if (ninten[i]!=0) 
                                ginten[i]=ginten[i]/(double)ninten[i];
                }
        }
  for (i=0;i<nx*ny*nz;i++){
    fout=(float)ginten[i];
    fwrite(&fout,sizeof(float),1,bfile);
  }
  fclose(bfile); 
  delete [] ginten;
        if (fmethod==1) {
                delete [] ninten;
        }
        else if (fmethod==2){
                delete [] distNearest;
        }
  return 0;
}
inline void getind(float xmsdel,float hwdel,int n, int &imin,
    int &imax)
{
  imin= (int)ceil(xmsdel-hwdel);
  if (imin<0) imin=0;
  if (imin>(n-1)) imin=n;
  imax= (int)floor(xmsdel+hwdel);
  if (imax<0) imax=-1;
  if (imax>(n-1)) imax=n-1;
}

int nentry(FILE *pfile)
{
  int c;
  int fslash=0;
  do {
    c=getc(pfile);
    if (c=='\\') {
      fslash=1;
      c=getc(pfile);
    }
    else fslash=0;
  } while(!(c=='$' && !fslash) && (c!=EOF));
  if (c==EOF)
    return 0;
  else
    return 1;
}
  
